@@ -1,6 +1,9 @@
 """Utility classes and functions for configuring and setting up the content and the look of a QR code."""
 import decimal
 from collections import namedtuple
+from datetime import date
+from typing import Optional, Any, Union, Tuple
+
 from django.utils.html import escape
 from qr_code.qrcode.constants import DEFAULT_MODULE_SIZE, SIZE_DICT, \
     DEFAULT_ERROR_CORRECTION, DEFAULT_IMAGE_FORMAT
@@ -10,17 +13,17 @@ class QRCodeOptions:
     """
     Represents the options used to create and draw a QR code.
     """
-    def __init__(self, *, size=DEFAULT_MODULE_SIZE, border=4, version=None,
-                 image_format='svg', error_correction=DEFAULT_ERROR_CORRECTION,
-                 micro=False, dark_color='#000', light_color='#fff',
-                 finder_dark_color=False, finder_light_color=False,
-                 data_dark_color=False, data_light_color=False,
-                 version_dark_color=False, version_light_color=False,
-                 format_dark_color=False, format_light_color=False,
-                 alignment_dark_color=False, alignment_light_color=False,
-                 timing_dark_color=False, timing_light_color=False,
-                 separator_color=False, dark_module_color=False, 
-                 quiet_zone_color=False):
+    def __init__(self, *, size: Union[int, str] = DEFAULT_MODULE_SIZE, border: int = 4, version: Union[int, str, None] = None,
+                 image_format: str = 'svg', error_correction: str = DEFAULT_ERROR_CORRECTION,
+                 micro: bool = False, dark_color: Union[Tuple, str] = '#000', light_color: Union[Tuple, str] = '#fff',
+                 finder_dark_color: bool = False, finder_light_color: bool = False,
+                 data_dark_color: bool = False, data_light_color: bool = False,
+                 version_dark_color: bool = False, version_light_color: bool = False,
+                 format_dark_color: bool = False, format_light_color: bool = False,
+                 alignment_dark_color: bool = False, alignment_light_color: bool = False,
+                 timing_dark_color: bool = False, timing_light_color: bool = False,
+                 separator_color: bool = False, dark_module_color: bool = False,
+                 quiet_zone_color: bool = False) -> None:
         """
         :param size: The size of the QR code as an integer or a string.
                     Default is *'m'*.
@@ -84,11 +87,11 @@ class QRCodeOptions:
         self._size = size
         self._border = border
         if _can_be_cast_to_int(version):
-            version = int(version)
+            version = int(version)  # type: ignore
             if not 1 <= version <= 40:
                 version = None
         elif version in ('m1', 'm2', 'm3', 'm4', 'M1', 'M2', 'M3', 'M4'):
-            version = version.lower()
+            version = version.lower()   # type: ignore
             # Set / change the micro setting otherwise Segno complains about
             # conflicting parameters
             micro = True
@@ -205,7 +208,7 @@ class QRCodeOptions:
         return self._micro
 
 
-def _can_be_cast_to_int(value):
+def _can_be_cast_to_int(value: Any) -> bool:
     return isinstance(value, int) or (isinstance(value, str) and value.isdigit())
 
 
@@ -228,36 +231,24 @@ class ContactDetail:
         * nickname: display name.
         * org: organization or company name (non-standard,but often recognized, ORG field).
     """
-    first_name = None
-    last_name = None
-    first_name_reading = None
-    last_name_reading = None
-    tel = None
-    tel_av = None
-    email = None
-    memo = None
-    birthday = None
-    address = None
-    url = None
-    nickname = None
-    org = None
 
-    def __init__(self, **kwargs):
-        self.first_name = kwargs.get('first_name')
-        self.last_name = kwargs.get('last_name')
-        self.first_name_reading = kwargs.get('first_name_reading')
-        self.last_name_reading = kwargs.get('last_name_reading')
-        self.tel = kwargs.get('tel')
-        self.tel_av = kwargs.get('tel_av')
-        self.email = kwargs.get('email')
-        self.memo = kwargs.get('memo')
-        self.birthday = kwargs.get('birthday')
-        self.address = kwargs.get('address')
-        self.url = kwargs.get('url')
-        self.nickname = kwargs.get('nickname')
-        self.org = kwargs.get('org')
+    def __init__(self, first_name: Optional[str] = None, last_name: Optional[str] = None, first_name_reading: Optional[str] = None, last_name_reading: Optional[str] = None, tel: Optional[str] = None, tel_av: Optional[str] = None, email: Optional[str] = None, memo: Optional[str] = None,
+                 birthday: Optional[date] = None, address: Optional[str] = None, url: Optional[str] = None, nickname: Optional[str] = None, org: Optional[str] = None):
+        self.first_name = first_name
+        self.last_name = last_name
+        self.first_name_reading = first_name_reading
+        self.last_name_reading = last_name_reading
+        self.tel = tel
+        self.tel_av = tel_av
+        self.email = email
+        self.memo = memo
+        self.birthday = birthday
+        self.address = address
+        self.url = url
+        self.nickname = nickname
+        self.org = org
 
-    def make_qr_code_text(self):
+    def make_qr_code_text(self) -> str:
         """
         Make a text for configuring a contact in a phone book. The MeCARD format is used, with an optional, non-standard (but often recognized) ORG field.
 
@@ -266,81 +257,79 @@ class ContactDetail:
         :return: the MeCARD contact description.
         """
 
-        _escape_mecard_special_chars_in_object_fields(self, ('first_name', 'last_name', 'first_name_reading', 'last_name_reading', 'tel', 'tel_av', 'email', 'memo', 'url', 'nickname', 'org'))
-
         # See this for an archive of the format specifications:
         # https://web.archive.org/web/20160304025131/https://www.nttdocomo.co.jp/english/service/developer/make/content/barcode/function/application/addressbook/index.html
         contact_text = 'MECARD:'
-        for name_components_pair in (('N:%s;', (self.escaped_last_name, self.escaped_first_name)), ('SOUND:%s;', (self.escaped_last_name_reading, self.escaped_first_name_reading))):
+        for name_components_pair in (
+                ('N:%s;', (_escape_mecard_special_chars(self.last_name), _escape_mecard_special_chars(self.first_name))),
+                ('SOUND:%s;', (_escape_mecard_special_chars(self.last_name_reading), _escape_mecard_special_chars(self.first_name_reading)))
+        ):
             if name_components_pair[1][0] and name_components_pair[1][1]:
                 name = '%s,%s' % name_components_pair[1]
             else:
-                name = name_components_pair[1][0] if name_components_pair[1][0] else name_components_pair[1][1]
+                name = name_components_pair[1][0] or name_components_pair[1][1] or ''
             if name:
                 contact_text += name_components_pair[0] % name
         if self.tel:
-            contact_text += 'TEL:%s;' % self.escaped_tel
+            contact_text += 'TEL:%s;' % _escape_mecard_special_chars(self.tel)
         if self.tel_av:
-            contact_text += 'TEL-AV:%s;' % self.escaped_tel_av
+            contact_text += 'TEL-AV:%s;' % _escape_mecard_special_chars(self.tel_av)
         if self.email:
-            contact_text += 'EMAIL:%s;' % self.escaped_email
+            contact_text += 'EMAIL:%s;' % _escape_mecard_special_chars(self.email)
         if self.memo:
-            contact_text += 'NOTE:%s;' % self.escaped_memo
+            contact_text += 'NOTE:%s;' % _escape_mecard_special_chars(self.memo)
         if self.birthday:
             # Format date to YYMMDD.
             contact_text += 'BDAY:%s;' % self.birthday.strftime('%Y%m%d')
         if self.address:
             contact_text += 'ADR:%s;' % self.address
         if self.url:
-            contact_text += 'URL:%s;' % self.escaped_url
+            contact_text += 'URL:%s;' % _escape_mecard_special_chars(self.url)
         if self.nickname:
-            contact_text += 'NICKNAME:%s;' % self.escaped_nickname
+            contact_text += 'NICKNAME:%s;' % _escape_mecard_special_chars(self.nickname)
         # Not standard, but recognized by several readers.
         if self.org:
-            contact_text += 'ORG:%s;' % self.escaped_org
+            contact_text += 'ORG:%s;' % _escape_mecard_special_chars(self.org)
         contact_text += ';'
         return contact_text
 
+    def escaped_value(self, field_name: str):
+        return _escape_mecard_special_chars(getattr(self, field_name))
 
-class WifiConfig(object):
+
+class WifiConfig:
     """
     Represents the configuration of a Wi-Fi connexion.
 
     The following fields are provided:
         * ssid: the name of the SSID
         * authentication: the authentication type for the SSID; can be AUTHENTICATION.wep or AUTHENTICATION.wpa, or AUTHENTICATION.nopass for no password. Or, omit for no password.
-        * password: the password, ignored if "authentsication" is 'nopass' (in which case it may be omitted).
+        * password: the password, ignored if "authentication" is 'nopass' (in which case it may be omitted).
         * hidden: tells whether the SSID is hidden or not; can be True or False.
     """
-    AUTHENTICATION = namedtuple('AUTHENTICATION', 'nopass WEP WPA')._make(range(3))
+    AUTHENTICATION = namedtuple('AUTHENTICATION', 'nopass WEP WPA')._make(range(3)) # type: ignore
     AUTHENTICATION_CHOICES = ((AUTHENTICATION.nopass, 'nopass'), (AUTHENTICATION.WEP, 'WEP'), (AUTHENTICATION.WPA, 'WPA'))
-    ssid = None
-    authentication = AUTHENTICATION.nopass
-    password = None
-    hidden = False
 
-    def __init__(self, **kwargs):
-        self.ssid = kwargs.get('ssid')
-        self.authentication = kwargs.get('authentication', WifiConfig.AUTHENTICATION.nopass)
-        self.password = kwargs.get('password')
-        self.hidden = kwargs.get('hidden')
+    def __init__(self, ssid: str = '', authentication: int = AUTHENTICATION.nopass, password: str = '', hidden: bool = False) -> None:
+        self.ssid = ssid
+        self.authentication = authentication
+        self.password = password
+        self.hidden = hidden
 
-    def make_qr_code_text(self):
+    def make_qr_code_text(self) -> str:
         """
         Make a text for configuring a Wi-Fi connexion. The syntax is inspired by the MeCARD format used for contacts.
 
         :return: the WIFI configuration text that can be translated to a QR code.
         """
 
-        _escape_mecard_special_chars_in_object_fields(self, ('ssid', 'password'))
-
         wifi_config = 'WIFI:'
         if self.ssid:
-            wifi_config += 'S:%s;' % self.escaped_ssid
+            wifi_config += 'S:%s;' % _escape_mecard_special_chars(self.ssid)
         if self.authentication:
             wifi_config += 'T:%s;' % WifiConfig.AUTHENTICATION_CHOICES[self.authentication][1]
         if self.password:
-            wifi_config += 'P:%s;' % self.escaped_password
+            wifi_config += 'P:%s;' % _escape_mecard_special_chars(self.password)
         if self.hidden:
             wifi_config += 'H:%s;' % str(self.hidden).lower()
         wifi_config += ';'
@@ -348,55 +337,49 @@ class WifiConfig(object):
 
 
 class Coordinates:
-    def __init__(self, latitude, longitude, altitude=None):
+    def __init__(self, latitude, longitude, altitude=None) -> None:
         self.latitude = latitude
         self.longitude = longitude
         self.altitude = altitude
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.altitude:
             return 'latitude: %s, longitude: %s, altitude: %s' % (self.latitude, self.longitude, self.altitude)
         return 'latitude: %s, longitude: %s' % (self.latitude, self.longitude)
 
-    def make_geolocation_text(coordinates):
+    def make_geolocation_text(self) -> str:
         return 'geo:%s,%s,%s' % (
-            escape(coordinates.latitude), escape(coordinates.longitude), escape(coordinates.altitude))
+            escape(self.latitude), escape(self.longitude), escape(self.altitude))
 
-    def make_google_maps_text(coordinates):
+    def make_google_maps_text(self) -> str:
         return 'https://maps.google.com/local?q=%s,%s' % (
-            escape(coordinates.latitude), escape(coordinates.longitude))
+            escape(self.latitude), escape(self.longitude))
 
 
-def make_email_text(email):
+def make_email_text(email: str) -> str:
     return 'mailto:%s' % email
 
 
-def make_tel_text(phone_number):
+def make_tel_text(phone_number: Any) -> str:
     return 'tel:%s' % phone_number
 
 
-def make_sms_text(phone_number):
+def make_sms_text(phone_number: Any) -> str:
     return 'sms:%s' % phone_number
 
 
-def make_youtube_text(video_id):
+def make_youtube_text(video_id: str) -> str:
     return 'https://www.youtube.com/watch/?v=%s' % escape(video_id)
 
 
-def make_google_play_text(package_id):
+def make_google_play_text(package_id: str) -> str:
     return 'https://play.google.com/store/apps/details?id=%s' % escape(package_id)
 
 
-def _escape_mecard_special_chars(string_to_escape):
+def _escape_mecard_special_chars(string_to_escape: Optional[str]) -> Optional[str]:
     if not string_to_escape:
         return string_to_escape
     special_chars = ['\\', '"', ';', ',', ':']
     for sc in special_chars:
         string_to_escape = string_to_escape.replace(sc, '\\%s' % sc)
     return string_to_escape
-
-
-def _escape_mecard_special_chars_in_object_fields(obj, keys):
-    for key in keys:
-        if hasattr(obj, key):
-            setattr(obj, 'escaped_%s' % key, _escape_mecard_special_chars(getattr(obj, key)))
