@@ -4,11 +4,25 @@ output_folder_name=tests_result
 if [ -d "../${output_folder_name}" ]; then
     mv "../${output_folder_name}" "../${output_folder_name}.back"
 fi
-mkdir "../${output_folder_name}"
+mkdir -p "../${output_folder_name}"
 log_file_name=log.txt
+
 (
     cd ../
 
+    echo "--- Run type checking"
+    pip install -r requirements-dev.txt
+    mypy qr_code
+    if [ $? -ne 0 ]; then
+        echo
+        echo --- Failed type checking. Abort tests!
+        exit 1
+    else
+        echo
+        echo --- Type checking OK!
+        echo
+    fi
+
     echo "--- Tests started on $(date)on $(hostname) ($(uname -a))"
     echo "--- Computer: $(hostname) ($(uname -a), CPU: $(nproc --all)"
     echo "--- CPU: $(nproc --all)"
@@ -32,7 +46,7 @@ log_file_name=log.txt
             ${DOCKER_COMPOSE_COMMAND} up -d
 
             echo --- Force Django version
-            ${DOCKER_COMPOSE_COMMAND} exec django-qr-code pip install "django~=${django_version}"
+            ${DOCKER_COMPOSE_COMMAND} exec django-qr-code pip install --upgrade "django~=${django_version}"
             echo Output code for tests with Python ${python_version} and Django ${django_version}: $?
 
             echo --- Setup test environment
